(* Definition of the CPS conversion algorithm *)

Require Import Coq.Lists.List.
Require Import "Variable".
Require Import SourceLang.
Require Import CpsLang.

Open Scope ctype_scope.


(* CPS conversion of term t *)
Fixpoint cps_term (t : sterm) : cterm :=
  match t with
  | st_bvar n =>
      ct_cabs (ct_app (ct_cbvar 0) (ct_sbvar n))
  | st_fvar x =>
      ct_cabs (ct_app (ct_cbvar 0) (ct_sfvar x))
  | st_unit =>
      ct_cabs (ct_app (ct_cbvar 0) ct_unit)
  | st_abs t1 =>
      ct_cabs (ct_app (ct_cbvar 0) (ct_sabs (cps_term t1)))
  | st_app tf ta =>
      ct_cabs (ct_app (cps_term tf)
        (ct_cabs (ct_app (cps_term ta)
           (ct_cabs (ct_app (ct_app (ct_cbvar 1) (ct_cbvar 0)) (ct_cbvar 2))))))
  end.

(* CPS conversion of type T *)
Fixpoint cps_type (T : stype) : ctype :=
  match T with
  | sy_unit =>
      cy_unit
  | sy_func TA TR =>
      cy_func (cps_type TA) (! ! cps_type TR)
  end.

(* CPS conversion of context C *)
Definition cps_ctxt : sctxt -> cctxt :=
  map (fun e : var * stype => let (x, T) := e in (x, cps_type T)).
