(* Theorems on lists *)

Require Import Coq.Arith.Lt.
Require Import Coq.Lists.List.


Lemma remove_remove_In :
  forall {A} (eq_dec : forall x y : A, {x=y}+{x<>y}) (l : list A) (a b : A),
  In a (remove eq_dec b l) ->
  In a l /\ a <> b.
Proof.
  induction l as [ | a' l' IH ]; intros a b H.
    case H.

    simpl in H. destruct (eq_dec b a') as [ H' | H' ].
      rewrite H' in *.
      pose proof (IH a a' H) as H1.
      split.
        apply in_cons, H1.
        apply H1.

      destruct H as [ H | H ].
        rewrite H in *.
        split.
          apply in_eq.
          congruence.

        pose proof (IH a b H) as H1.
        split.
          apply in_cons, H1.
          apply H1.
Qed.

Lemma neq_remove_In :
  forall {A} (eq_dec : forall x y : A, {x=y}+{x<>y}) (l : list A) (a b : A),
  a <> b ->
  In a l ->
  In a (remove eq_dec b l).
Proof.
  induction l as [ | x' l' IH ]; intros a b H1 H2.
    case H2.

      simpl; destruct (eq_dec b x') as [ H3 | H3 ].
        apply IH; clear IH.
          exact H1.

          rewrite H3 in *.
          destruct H2 as [ H2 | H2 ]; [ congruence | exact H2 ].

        destruct H2 as [ H2 | H2 ].
          left; rewrite H2; reflexivity.

          right; apply IH; [ exact H1 | exact H2 ].
Qed.


Fixpoint in' {A B : Type} (a : A) (b : B) (l : list (A * B)) : Prop :=
  match l with
  | nil =>
      False
  | (a', b') :: tl =>
      a' = a /\ b' = b \/ a' <> a /\ in' a b tl
  end.
